@import url("stylesheet.css");

#panel.bottom-panel {
    border-top-width: 1px;
    border-bottom-width: 0px;
    height: 2.25em ;
  }

  .bottom-panel .window-button > StWidget {
    background-gradient-drection: vertical;
    background-color: #fff;
    background-gradient-start: #fff;
    background-gradient-end: #eee;
    color: #000;
    border-radius: 2px;
    padding: 4px 6px 2px;
    text-shadow: 0 0 transparent;
    box-shadow: inset -1px -1px 1px rgba(0,0,0,0.5);
  }

  .bottom-panel .window-button:hover > StWidget {
    background-color: #fff;
    background-gradient-start: #fff;
    background-gradient-end: #fefefe;
  }

  .bottom-panel .window-button:active > StWidget {
    box-shadow: inset 1px 1px 2px rgba(0,0,0,0.5);
    background-color: #ededed;
    background-gradient-start: #ededed;
    background-gradient-end: #ededed;
  }

  .bottom-panel .window-button.focused > StWidget {
    background-color: #ddd;
    background-color: #ddd;
    background-gradient-end: #dedede;
    color: white;
    box-shadow: inset 1px 1px 2px rgba(0,0,0,0.5);
  }

  .bottom-panel .window-button.focused:hover > StWidget {
    background-color: #b9b9b9;
    background-gradient-end: #c0c0c0;
  }

  .bottom-panel .window-button.minimized > StWidget {
    color: #000;
    box-shadow: inset -1px -1px 1px rgba(0,0,0,0.4);
  }

